# proscan-lib.pl
# Functions for configuring ProScan Anti-Virus for Mailserver
use Time::tm;

do '../web-lib.pl';

&init_config();

$section_row_bgcolor = $cb;

@date_fmt =(
	{fmt => "%d", name => $text{'gnrl_DD'}},
	{fmt => "%m", name => $text{'gnrl_MM'}},
	{fmt => "%Y", name => $text{'gnrl_YYYY'}},
	{fmt => "%y", name => $text{'gnrl_YY'}}
);

$notify_section = "smtpscan.notify";

@notify_user = (
	{user => 'admin', name => $text{'group_notify_admin'}},
	{user => 'recipient', name => $text{'group_notify_rcpt'}},
	{user => 'sender', name => $text{'group_notify_sndr'}},
);

@notify_type = (
	 {type => "infected", name => $text{'group_notify_inf'}},
	 {type => "other", name => $text{'group_notify_oth'}},
	 {type => "suspicious", name => $text{'group_notify_sus'}},
	 {type => "filtered", name => $text{'group_notify_fltr'}},
	 {type => "error", name => $text{'group_notify_err'}},
	 {type => "protected", name => $text{'group_notify_prot'}},
);

%proscan_errors=(
	 0 => $text{'aveserver_error_0'},
	 1 => $text{'aveserver_error_1'},
	 2 => $text{'aveserver_error_2'},
	 3 => $text{'aveserver_error_3'},
	 4 => $text{'aveserver_error_4'},
	 5 => $text{'aveserver_error_5'},
	 6 => $text{'aveserver_error_6'},
	 7 => $text{'aveserver_error_7'},
	 8 => $text{'aveserver_error_8'},
	 9 => $text{'aveserver_error_9'},
	 11 => $text{'aveserver_error_11'});

%scanner_errors=(
	 0 => $text{'scanner_error_0'},
	 1 => $text{'scanner_error_1'},
	 2 => $text{'scanner_error_2'},
	 3 => $text{'scanner_error_3'},
	 4 => $text{'scanner_error_4'},
	 5 => $text{'scanner_error_5'},
	 99 => $text{'scanner_error_99'});

%updater_errors=(
	 0 => $text{'updater_error_0'},
	 1 => $text{'updater_error_1'},
	 2 => $text{'updater_error_2'},
	 3 => $text{'updater_error_3'},
	 4 => $text{'updater_error_4'},
	 5 => $text{'updater_error_5'},
	 6 => $text{'updater_error_6'},
	 7 => $text{'updater_error_7'},
	 8 => $text{'updater_error_8'},
	 9 => $text{'updater_error_9'},
	 11 => $text{'updater_error_11'},
	 99 => $text{'updater_error_99'});


@binaries=(
	{disp       => "wm_ave.log",
	 pid        => "$config{'tmp_exec_path'}/wm_ave.pid",
	 bin        => $config{'ave_path'},
	 name       => $text{'ave_module'},
	 errors     => \%proscan_errors,
	 cfg        => "ave_edit.cgi",
	 descr      => $text{'ave_descr'},
	 extra_cmd  => "",
	 start_cmd  => $config{'proscan_path'},
 	 stop_cmd   => "$config{'proscan_path'} stop",
	 log_sec    => "aveserver",
	 log_name   => "ReportFileName",
	 log_append => "Append",
	 proscanav_pid_dir => "$config{'tmp_exec_path'}/../run"},
	 
	{disp       => "wm_scanner.log",
	 pid        => "$config{'tmp_exec_path'}/wm_scanner.pid",
	 bin        => "$config{'scanner_path'}",
	 name       => $text{'scan_module'},
	 errors     => \%scanner_errors,
	 cfg        => "file_scan_edit.cgi",
	 descr      => $text{'scan_descr'},
	 start_redir => "file_scan_start.cgi",
	 extra_cmd  => "",
	 start_cmd  => $config{'scanner_path'},
 	 stop_cmd   => $config{'scanner_path'},
	 log_sec    => "scanner.report",
	 log_name   => "ReportFileName",
	 log_append => "Append"},
	 
	{disp       => "wm_updater.log",
	 pid        => "$config{'tmp_exec_path'}/wm_updater.pid",
	 bin        => $config{'updater_path'},
	 name       => $text{'updater_module'},
	 errors     => \%updater_errors,
	 cfg        => "updater_edit.cgi",
	 descr      => $text{'updater_descr'},
	 extra_cmd  => "",
	 start_cmd  => $config{'updater_path'},
 	 stop_cmd   => $config{'updater_path'},
	 log_sec    => "updater.report",
	 log_name   => "ReportFileName",
	 log_append => "Append"}
);

@charset_types=("","US-ASCII","ISO-2022-JP","EUC-JP","Shift_JIS");
%kcode=("US-ASCII"=>"euc","ISO-2022-JP"=>"jis","EUC-JP"=>"euc","Shift_JIS"=>"sjis");

sub get_textarea_list{
    my( $str, $delimiter ) = @_;
    $str=~s/\x0D//g;
    $str=~s/^[\x20\x09]*?$//mg;
    $str=~s/\n\Z//mg;
    $str=~s/\n/$delimiter/g;
    return $str;
}

sub read_config_file {
    my($HANDLE,$fname) = @_;
    my(@data) = ();
	while(<$HANDLE>){
		s/(\x0D\x0A|\x0D|\x0A)//;
		next if /^#/ || /^$/;
	    if ( /_include (.*)/i ) {
	        $include = $1;
	        if ( $include =~ /^\./ ) {
	            ($path,$file) = $fname =~ /(.*)\/(.*)/;
	            $include = "$path/$include";
	        }
            open(INC,"$include") or die "Cannot open include file($include): $!";
	        my $config = read_config_file(INC,$include);
            close(INC);
	        push(@data,@{$config});
	        next;
	    }
	    push(@data,$_);
	}
	return \@data;
}

sub get_config{
	$proscan_config=$config{'proscan_conf'};

	%proscan_conf;
	open(CONFIG,"$proscan_config") or die "Cannot open configuraton file($proscan_config): $!";
	$config_data = &read_config_file(CONFIG,$proscan_config);
	close(CONFIG);
#debug
#	open(CONFIG,">$proscan_config.debug") || &error("Can't open '$proscan_config.debug'.");
#    foreach $line (@{$config_data}) {
#        print CONFIG "$line\n";
#    }
#	close(CONFIG);
#debug
	$group="common";
	@group_list = ();
	$order=0;
	foreach $line (@{$config_data}) {
		if ($line=~/^\[(.*)\]$/){
			$sec=$1;
			$proscan_conf{$group}{$sec}{"order"}=$order;
#print qq(\$proscan_conf{$group}{$sec}{"order"}=$order<br>\n);
			$order++;
		}
		elsif ($line=~/(.*?)\=(.*)/){
			$key=$1;
			$val=$2;
			$proscan_conf{$group}{$sec}{"content"}{$key}=$val;
#print qq(\$proscan_conf{$group}{$sec}{"content"}{$key}=$val<br>\n);
		}
		elsif ($line=~/_group (.*)/){
			$group=$1;
			push(@group_list,$group);
		}
		elsif ($line=~/_end_group/){
			$group="common";
		}
	}
	return \%proscan_conf;
}

sub increase_order
{
	my ($proscan_conf,$first_index,$group) = @_;
	foreach $sect (keys %{$proscan_conf}){
		if( $proscan_conf->{$group}{$sect}{"order"}>=$first_index ){
			$proscan_conf->{$group}{$sect}{"order"}=$proscan_conf->{$group}{$sect}{"order"}+1;
		}
	}
}


sub add_section{
	my( $proscan_config, $new_component_name, $new_sec_name, $new_sect, $add_to_begin ) = @_;

	$first_index = -1;
	$last_index = -1;
	
	foreach $sect (sort { $proscan_conf->{$a}{"order"} <=> $proscan_conf->{$b}{"order"} } keys %{$proscan_config}){
		if( $sect =~ /^$new_component_name[.:].*?/ ){
			if( $first_index==-1 ){
				$first_index=$proscan_conf->{$sect}{"order"};
			}
			$last_index=$proscan_conf->{$sect}{"order"};
		}	
	}

	if( $add_to_begin!=0 ){
		if($first_index==-1){#add to end
			while (($k, $v) = each %{$new_sect}) {
				$proscan_conf->{$new_sec_name}{"content"}{$k} = $v;
			} 
		}else{
			increase_order($proscan_config, $first_index);
			$proscan_conf->{$new_sec_name}{"order"} = $first_index;
			while (($k, $v) = each %{$new_sect}) {
				$proscan_conf->{$new_sec_name}{"content"}{$k} = $v;
			} 
		}

	}else{
		if($last_index==-1){#add to end
			while (($k, $v) = each %{$new_sect}) {
				$proscan_conf->{$new_sec_name}{"content"}{$k} = $v;
			} 
		}else{
			increase_order($proscan_config,$last_index+1);
			$proscan_conf->{$new_sec_name}{"order"} = $last_index+1;
			while (($k, $v) = each %{$new_sect}) {
				$proscan_conf->{$new_sec_name}{"content"}{$k} = $v;
			} 
		}

	}

}

sub put_config_data {
    my( $group,$conf,$OUT ) = @_;
    $section_list = $conf->{$group};
	foreach $sect (sort { $conf->{$group}{$a}{"order"} <=> $conf->{$group}{$b}{"order"} } keys %{$section_list}){
		if( $sect ne "" ){
			print $OUT "[$sect]\n";
		}
		foreach $key (sort keys %{$conf->{$group}{$sect}{"content"}}) {
			if( $key ne "") {
			    $val = $conf->{$group}{$sect}{"content"}{$key};
				print $OUT "$key=$val\n" if $val ne "";
			}
		}
		print $OUT "\n";
	}
}

sub put_config{
	my( $conf ) = @_;
	$proscan_config=$config{'proscan_conf'};

    system("/bin/cp $proscan_config $proscan_config.back");
	open(CONFIG,">$proscan_config") || &error("Can't open '$proscan_config'.");
    put_config_data("common",$conf,CONFIG);
    foreach $group (@group_list) {
        print CONFIG "_group $group\n\n";
        put_config_data($group,$conf,CONFIG);
    }
    print CONFIG "_end_group\n\n";
	close(CONFIG);
}


sub to_bool{
	my( $val ) = @_;
	if( $val=~/^y|^yes|^1/i ){
		return 1;
	}
	return 0;
}

sub get_bool{
	my($in, $name)=@_;
	return (!$in->{$name}||$in->{$name}!=1) ? 'no' : 'yes';
}

sub print_table_header{

my( $tb_title, $tb_hlp) = @_;
print "<table border=0 cellspacing=1 cellpadding=3 width=100% bgcolor=#FFFFFF>\n";
print "	<tr $tb> <td nowrap colspan=3>\n";
print "		<table width=100% border=0 cellpadding=0 cellspacing=0>\n";
print "		<tr><td><font color=#FFFFFF face=Arial size=-1><b>$tb_title</b></font></td>\n";
$hlp_ref="$gconfig{'webprefix'}/help.cgi/$module_name/help".$tb_hlp;
print "		<td align=right><a onClick='window.open(\"$hlp_ref\", \"help\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300,resizable=yes\"); return false'";
print "href=\"$hlp_ref\" alt=\"help\"><font color=#FFFFFF face=Arial size=-1><b>?</b></font></a></td></tr>\n";
print "		</table>\n";
print "	</td></tr>\n";
}

sub print_table_footer{
#print "	</td></tr>\n";
print "</table>\n";
}

sub print_opt_name{

	my( $opt_name ) = @_;
	$opt_name=~s/[\r\n]//;
	print "<font size=-1>$opt_name</font>\n";
}

sub print_checkbox{

	my( $var_name, $cheked, $bit ) = @_;
	$cheked=to_bool($cheked);
	print "<input type=checkbox name=$var_name value=1";
	if( $cheked==1 ){
		print " checked";
	}
	print ">\n";
}

sub print_bit_checkbox{

	my( $text_name, $var_name, $num, $bit ) = @_;
	$checked=($num & $bit)!=0?1:0;
	$checked=to_bool($checked);
	print "<input type=checkbox name=$var_name value=$bit";
	if( $checked==1 ){
		print " checked";
	}
	print ">$text_name\n";
}


sub print_editbox{

	my( $name, $size, $val ) = @_;
	print "&nbsp;<input name=$name size=$size value=\"$val\">\n";
}


sub print_select{
	my( $var_name, $size, $values, $titles, $select) = @_;
	print "&nbsp;<SELECT size=$size name=$var_name>\n";
	for ($i = 0; $i < @$values; $i++) {
		print "	<OPTION value=$values->[$i]";
		if( $values->[$i] eq $select ){
			print(" selected");
		}
		print ">$titles->[$i]\n";
	}
	print "</SELECT>\n";
}

# my_file_chooser_button(input, type, [form], [chroot])
# Return HTML for a file chooser button, if the browser supports Javascript.
# Type values are 0 for file or directory, or 1 for directory only
# Append new val to old unlike original function
sub my_file_chooser_button
{
local $form = @_ > 2 ? $_[2] : 0;
local $chroot = @_ > 3 ? $_[3] : "/";
return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"app_chooser.cgi?type=$_[1]&chroot=$chroot&file=\"+escape(ifield.value), \"chooser\", \"toolbar=no,menubar=no,scrollbar=no,width=400,height=300\"); chooser.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
}

sub set_run_info{

	my($c_id, $pid, $status, $log_offset) = @_;
#	$path = $config{'tmp_exec_path'};
#	$path =~s/\/$//;
	$pid_file = $binaries[$c_id]{pid};
	if (-e $pid_file){
		open(FH,"$pid_file") or die "Cannot open pid file: $!";
		$old_pid=<FH>;
		$old_pid=~s/\n//;
		close FH;
		if($old_pid!=-1){
			return 0 if kill 0 => $old_pid ;
			return 0 if !(-w $pid_file && unlink $pid_file) ;
		}
	}
	open(FH,">$pid_file") or return 0;
	print FH "$pid\n";
	print FH "$status\n";
	$time =  localtime();
	print FH "$time\n";
	print FH "$log_offset";
	close FH;
	return 1;
}

sub check_pid{

	my($c_id) = @_;

	my $ps_pattern = $binaries[$c_id]{bin};

    $os = `uname`;
    $ps_cmd = ($os =~ /SunOS/) ? "ps -ef" : "ps -ax";
    open(PS,"$ps_cmd|");
    while(<PS>) {
        if ( $c_id == 0 ) { #engine
	        if(/$ps_pattern/) {
	            next if /--disable-proxy/;
	            ($pid) = /^(\d+) /;
	            return $pid;
	        }
        } elsif ( $c_id == 1 ) { #filescanner
	        if(/$ps_pattern/) {
	            ($pid) = /^(\d+) /;
	            return $pid;
	        }
        } elsif ( $c_id == 2 ) { #updater
	        if(/$ps_pattern/) {
	            ($pid) = /^(\d+) /;
	            return $pid;
	        }
	    }
    }
	return 0;
}

sub get_run_info
{
	my($c_id) = @_;
	$pid_file = $binaries[$c_id]{pid};
	if (-e $pid_file){
		open(FH,"$pid_file") or die "Cannot open pid file: $!";
		$old_pid=<FH>;
		$old_pid=~s/\n//;
		$status=<FH>;
		$status=~s/\n//;
		$time=<FH>;
		$time=~s/\n//;
		$log_offset=<FH>;
		$log_offset=~s/\n//;
		close FH;
		@ret_stat = ($old_pid, $status, $time, $log_offset);
		return @ret_stat;
	}
	return;
}

sub get_log_offset
{
	my($log, $append) = @_;

	if( !to_bool($append) ){
		return 0;
	} else {
		if(-e $log){
			&lock_file($log);
			open(FH,"$log") or die "Cannot open log file: $!";
			seek FH,0,2;
			$offset = tell FH;
			close FH;
			&unlock_file($log);
			return $offset;
		}
	}
	return 0;
}
